/* -*-c++-*-
 * 
 *    Source     $RCSfile: in_mc.cc,v $ 
 *    Version    $Revision: 1.1.1.1.2.1 $ 
 *    Date       $Date: 2000/02/28 13:55:47 $ 
 *    Author     Christoph Helmberg 
 * 
 --------------------------------------------------------------------------- */

// ****************************************************************************
//
//  This file is part of the class library "SBmethod", an implementation 
//  of the spectral bundle method for eigenvalue optimization.
//
//  Copyright (C) 2000  Christoph Helmberg
//
//     Konrad-Zuse-Zentrum fuer Informationstechnik Berlin
//     Takustrasse 7, D-14195 Berlin, Germany
//      
//     helmberg@zib.de     http://www.zib.de/helmberg 
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// ****************************************************************************

#ident "$Id: in_mc.cc,v 1.1.1.1.2.1 2000/02/28 13:55:47 bzfhelmb Exp $"

#include <stdio.h>
#include <stdlib.h>
#include "string.h"
#include "sparssym.h"
#include "cmsymspa.h"
#include "cmsymden.h"
#include "cmrankss.h"
#include "cmranksd.h"
#include "cmrankdd.h"
#include "cmgramsp.h"
#include "cmgramde.h"
#include "cmsingle.h"
#include <fstream.h>

//transforms a weighted graph description frm standard in
//(output format of rudy: n m i1 j1 w1 ... im jm wm )
//into an eigenvalue problem readable by the spectral bundle algorithm
//
//input indices are assumed to start at 1 and are decremented to start at 0
//if no decrement of the indices is desired the option -nd must be given
//in the command line

int message(const char *s,int i)
{
 cerr<<s<<endl;
 return i;
}

    
main(int argc,char **argv)
{
 int do_decrement=1;
 int ct=0;
 int equi=0;
 double scaleC=1.;
 const char *usage="\n usage: in_mc [-nd] [-ct Integer] [-sC real] -eq\n   -nd ... do not subtract 1 from index\n   -ct ... select singleton(0) symspa(1) gramspa(2) rankss(3) symden(4) gramden(5) ranksd(6) rankdd(7)\n   -sC ... scale the input matrix by multiplying with this number\n   -eq ... solve as equipartition problem";
 Integer i;
 for(i=1;i<argc;i++){
     if (strcmp(argv[i],"-nd")==0){
         do_decrement=0;
         continue;
     }
     if (strcmp(argv[i],"-ct")==0){
         i++;
         if (i>=argc) return message(usage,1);
         sscanf(argv[i],"%d",&ct);
         if ((ct<0)||(ct>7)) return message(usage,1);
         continue;
     }
     if (strcmp(argv[i],"-sC")==0){
         i++;
         if (i>=argc) return message(usage,1);
         sscanf(argv[i],"%lf",&scaleC);
         continue;
     }
     if (strcmp(argv[i],"-eq")==0){
         equi=1;
         continue;
     }
     return message(usage,1);
 }
 Integer n,m;
 cin>>n>>m;
 Indexmatrix indi(m,Integer(1),Integer(0));
 Indexmatrix indj(m,Integer(1),Integer(0));
 Matrix val(m,Integer(1),0.);
 if (equi) scaleC*=-1.;
 for(i=0;i<m;i++){
     cin>>indi(i)>>indj(i)>>val(i);
     val(i)*=scaleC;
     if (do_decrement){
         indi(i)--;
         indj(i)--;
     }
 }
 Sparsesym L(n,m,indi,indj,val);
 Matrix Ldiag(diag(L));
 L-=sparseDiag(Ldiag);
 Ldiag.init(n,1,sum(L)/Real(n));
 L*=-1;
 L+=sparseDiag(Ldiag);
 L/=4.;
 cout<<setprecision(20);
 cout<<n<<"\n";
 CMsymsparse cm(L);
 cm.out(cout);
 if (equi) cout<<n+1<<'\n';
 else cout<<n<<'\n';
 for(i=0;i<L.rowdim();i++){
   switch(ct){
   case 0:{
     CMsingleton cm(n,i,i,1.);
     cm.out(cout);
     break;
   }
   case 1: {
     CMsymsparse cm(Sparsesym(n,Integer(1),Indexmatrix(1,1,i),Indexmatrix(1,1,i),Matrix(1,1,1.)));
     cm.out(cout);
     break;
   }
   case 2:{
     CMgramsparse cm(Sparsemat(n,Integer(1),Integer(1),Indexmatrix(1,1,i),Indexmatrix(1,1,Integer(0)),Matrix(1,1,1.)));
     cm.out(cout);
     break;
   }
   case 3:{
     CMlowrankss cm(Sparsemat(n,Integer(1),Integer(1),Indexmatrix(1,1,i),Indexmatrix(1,1,Integer(0)),Matrix(1,1,1.)),Sparsemat(n,Integer(1),Integer(1),Indexmatrix(1,1,i),Indexmatrix(1,1,Integer(0)),Matrix(1,1,.5)));
     cm.out(cout);
     break;
   }
   case 4: {
     Symmatrix S(n,0.); S(i,i)=1;
     CMsymdense cm(S);
     cm.out(cout);
     break;
   }
   case 5:{
     Matrix vec(n,1,0.); vec(i)=1.;
     CMgramdense cm(vec);
     cm.out(cout);
     break;
   }
   case 6:{
     Matrix vec(n,1,0.); vec(i)=.5;
     CMlowranksd cm(Sparsemat(n,Integer(1),Integer(1),Indexmatrix(1,1,i),Indexmatrix(1,1,Integer(0)),Matrix(1,1,1.)),vec);
     cm.out(cout);
     break;
   }
   case 7:{
     Matrix vec1(n,1,0.); vec1(i)=1.;
     Matrix vec2(n,1,0.); vec2(i)=.5;
     CMlowrankdd cm(vec1,vec2);
     cm.out(cout);
     break;
   }
   
   }
   cout<<EQU<<' ';
   cout<<1.<<'\n';
 }
 if (equi) {
   Matrix vec(n,1,1.);
   CMgramdense cm(vec);
   cm.out(cout);
   cout<<EQU<<' ';
   cout<<0.<<'\n';
 }
     
 return 0;
}

/* ---------------------------------------------------------------------------
 *    Change log $Log: in_mc.cc,v $
 *    Change log Revision 1.1.1.1.2.1  2000/02/28 13:55:47  bzfhelmb
 *    Change log Reording inline functions and correction of minor details
 *    Change log
 *    Change log Revision 1.1.1.1  1999/12/15 13:47:52  bzfhelmb
 *    Change log Imported sources
 *    Change log
 *
 *    End of $Source: /kombadon/cvs/cvsroot/bzfhelmb/SBmethod/sb/in_mc.cc,v $
 --------------------------------------------------------------------------- */
