/* -*-c++-*-
 * 
 *    Source     $RCSfile: solver.cc,v $ 
 *    Version    $Revision: 1.1.1.1 $ 
 *    Date       $Date: 1999/12/15 13:47:52 $ 
 *    Author     Christoph Helmberg 
 * 
 --------------------------------------------------------------------------- */

// ****************************************************************************
//
//  This file is part of the class library "SBmethod", an implementation 
//  of the spectral bundle method for eigenvalue optimization.
//
//  Copyright (C) 2000  Christoph Helmberg
//
//     Konrad-Zuse-Zentrum fuer Informationstechnik Berlin
//     Takustrasse 7, D-14195 Berlin, Germany
//      
//     helmberg@zib.de     http://www.zib.de/helmberg 
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// ****************************************************************************

#ident "$Id: solver.cc,v 1.1.1.1 1999/12/15 13:47:52 bzfhelmb Exp $"

#include "solver.h"

int Solver::solve(Problem* problem,SpectralBundle* bundle,
                  Clock* myclock,SBparams* params,int resume)
{
 Integer n,m;
 Matrix eigval,eigvec,y;

 if (!resume){
     //-- initialization
     n=problem->mat_dim();
     m=problem->nr_constraints();
     bundle->init_size(n,m);
     if (out) {
         (*out)<<"dim="<<n;
         (*out)<<"  nr_constraints="<<m<<endl;
     }
     
     //---- compute maximal eigenvalues and eigenvectors of C
     Matrix eigval,eigvec;
     Matrix y;
     problem->get_y(y);
     Integer nreig=1;
     problem->obj_compute(y,eigval,eigvec,1e20,1e-5,nreig); 
     problem->set_y(y,eigval,eigvec);
     
     if (out) {
         (*out).setf(ios::showpoint);
         (*out)<<"\n"<<*myclock<<" endit ";out->width(2);(*out)<<0;
         out->precision(5);
         (*out)<<" ";out->width(3);(*out)<<bundle->get_cntobjeval()+1;
	 (*out)<<" ";out->width(8);(*out)<<(problem->get_eigval())(0);
         out->precision(8);
         (*out)<<" (";out->width(9);(*out)<<problem->objval();
	 out->precision(6);
	 if (bundle->get_lower_bound()>min_Real){
	   (*out)<<">";
	   (*out)<<bundle->get_lower_bound();
	 }
	 (*out)<<")"<<endl;
     }
 }
     
 //---- LOOP

 do{

     //---- call bundle algorithm
     bundle->inner_loop(*problem,resume);
     resume=0;
     
     //--- output
     if (out) bundle->print_line_summary((*out));

 } while(
     (!bundle->get_terminate())
     &&
     ((params->maxiter<0)||(bundle->get_calls()<params->maxiter))
     &&
     (myclock->time()<bundle->get_terminator()->get_timelimit())
    );

 return 0;
}

/* ---------------------------------------------------------------------------
 *    Change log $Log: solver.cc,v $
 *    Change log Revision 1.1.1.1  1999/12/15 13:47:52  bzfhelmb
 *    Change log Imported sources
 *    Change log
 *
 *    End of $Source: /kombadon/cvs/cvsroot/bzfhelmb/SBmethod/sb/solver.cc,v $
 --------------------------------------------------------------------------- */
